Module: Puppet::Pops::Validation Private
- Defined in:
- lib/puppet/pops/validation.rb,
lib/puppet/pops.rb,
lib/puppet/pops/validation/checker4_0.rb,
lib/puppet/pops/validation/tasks_checker.rb,
lib/puppet/pops/validation/validator_factory_4_0.rb
Overview
This module is part of a private API. You should avoid using this module if possible, as it may be removed or be changed in the future.
A module with base functionality for validation of a model.
- Factory - an abstract factory implementation that makes it easier to create a new validation factory.
- SeverityProducer - produces a severity (:error, :warning, :ignore) for a given Issue
- DiagnosticProducer - produces a Diagnostic which binds an Issue to an occurrence of that issue
- Acceptor - the receiver/sink/collector of computed diagnostics
- DiagnosticFormatter - produces human readable output for a Diagnostic
Defined Under Namespace
Classes: Acceptor, Checker4_0, Diagnostic, DiagnosticFormatter, DiagnosticFormatterPuppetStyle, DiagnosticProducer, Factory, SeverityProducer, TasksChecker, ValidatorFactory_4_0